function [r,x] = adslout2(h,sx,sn,iter,b)
% [r,x] = adslout2(h,sx,sn,iter,b,nflag)
%     r = received data sequence (noisy channel output)
%     x = transmitted data sequence (channel input)
%     h = transmission channel, unit norm assumed
%     ssx = input signal power
%     ssn = noise power
%     nflag = noise flag:
%        0 -> white noise with total power ssn
%        1 -> NEXT (upstream to downstream) with total power ssn
%     iter = number of symbols to generate
%
%     b = 256-vector containing bit rate assignment
%     If b is not given, 4-QAM constellations are used per carrier
%     [this is not actually used in the current version]
%
%     generates ADSL-like data
%     (not yet 100% standard compliant)
%     Created by Rick Martin on 05/20/2002
%     Last updated on 12/09/2002
%     Cornell University Blind Equalization Research Group
%
%     Portions of this code were obtained from the DMT-TEQ
%     toolbox, version 3.1, provided by The University of
%     Texas at Austin.
%
%     Copyright 2002 Cornell University, All Rights Reserved
%
%     SOFTWARE shall mean "adslout.m", Cornell Research Foundation
%     ("CRF") Docket D-3006.  SOFTWARE is made available to allow
%     non-commercial and research use of SOFTWARE.  Cornell
%     University specifically reserves all commercial rights to
%     SOFTWARE and these rights may be licensed by CRF on behalf
%     of Cornell University to third parties.
%
%     CRF provides SOFTWARE on an ``as is'' basis.  CRF does not
%     warrant, guarantee, or make any representations regarding
%     the use or results of SOFTWARE with respect to its
%     correctness, accuracy, reliability or performance.  The
%     entire risk of the use and performance of SOFTWARE is
%     assumed by EVALUATOR.  ALL WARRANTIES INCLUDING, WITHOUT
%     LIMITATION, ANY WARRANTY OF FITNESS  FOR A PARTICULAR
%     PURPOSE OR MERCHANTABILITY ARE HEREBY EXCLUDED.

nflag=0;

N = 512;   % FFT size
nu = 32;   % CP length
S = N+nu;  % symbol length

if nargin < 6, b = 2*ones(N,1); end;

% 4-QAM data
X = sx/sqrt(2)*( 2*floor(2*rand(N/2,iter))-1 ) + ...
  j*sx/sqrt(2)*( 2*floor(2*rand(N/2,iter))-1 );

% Take IFFT, convert P/S, and add CP
x = zeros(1,S*iter);
datain = zeros(N,1);
for k=1:iter
   %datain([2:27,39:64]) = X(:,k);
   datain = i2r(X(:,k).');
   x(S*(k-1)+[nu+1:S]) = real(ifft(datain));
   x(S*(k-1)+[1:nu]) = x(S*(k-1)+[N+1:S]);  % copy into CP
end

if nflag == 1
%-------------
% obtained from "dsl.m" in the DMT-TEQ toolbox version 3.1
% provided by The University of Texas at Austin
users = 8;
fsample = 2.208e6;
MM = 128;
f = linspace(0,fsample/2,MM);
Kmask = zeros(size(f));
Kmask(f>=28e3 & f<138e3) = 113e3*0.1*10^(-38/10);
Kmask(f>138e3) = 113e3*0.1*10.^( (-38-24*(f(f>138e3) - 138e3)/43125) /10);

fo = 276e3;
PSDadsl = Kmask.*sinc(f./fo).^2;
Hnext = PSDadsl .* f.^(3/2);
fnormed = f./max(f);
b = remez(128,fnormed,sqrt(Hnext));
a = 1;
NEXTinput = randn(S*iter,1);
NEXT = filter(b,a,NEXTinput);
NEXT = (sn/sqrt(var(NEXT))) * NEXT;
noise = NEXT.';
%-------------
else % plain old white noise
   noise = sn*randn(1,S*iter);
end % if nflag

% received signal
r = conv(x,h);
r = r(1:S*iter) + noise;
