function bmap = bitrate(c,w,nu,dopt,ssx,ssn)
% bmap = bitrate(c,w,nu,dopt,ssx,ssn)
%     c = channel
%     w = TEQ
%     nu = cyclic prefix length
%     dopt = starting index of window - 1
%     ssx = signal power
%     ssn = noise power
%
%     Calculates the achievable bit rate for
%     a given ADSL transmission environment
%
%     Modified from code in the DMT-TEQ toolbox,
%     provided by the University of Texas at Austin, ESPL
%
%     Modifications made by Rick Martin, February 2002
%     Last updated 06/20/2002
%     Cornell University Blind Equalization Research Group

margin = 6;         % dB
codingGain = 4.2;   % dB
snrgap = 9.8;       % dB
N = 512;            % FFT size
minb = 0;           % minimum number of bits per tone
maxb = 100;         % maximum number of bits per tone
qflag = 1;          % 1: bmap is integer valued

inputSpec = ssx * ones(1,N/2+1);
noiseSpec = ssn * ones(1,N/2+1);

win = zeros(length(c),1);
win(dopt+1:dopt+nu+1) = ones(1,nu+1);

% signal path impulse response
hwin = c.*win;
hwin = hwin - mean(hwin);  % ?

% ISI path impulse response
hwall = c.*(1-win);
hwall = hwall - mean(hwall);  % ?

% frequency responses
Fwu = fft(w,N)*sqrt(N);         Fw = Fwu(1:N/2+1).';
Fhwinu = fft(hwin,N)*sqrt(N);   Fhwin = Fhwinu(1:N/2+1).';
Fhwallu = fft(hwall,N)*sqrt(N); Fhwout = Fhwallu(1:N/2+1).';

% noise power spectrum
colorNoiseaft = noiseSpec.*abs(Fw).^2;

% ISI power spectrum
isiaft = inputSpec.*abs(Fhwout).^2;

% signal power spectrum
signalaft = inputSpec.*abs(Fhwin).^2;

% subband SNRs after equalization (as a vector)
SNRi = signalaft./( colorNoiseaft + isiaft );

bmap = ba_cal(SNRi,9.8,margin,codingGain,minb,maxb,qflag);
