function bmap = mfb(c,ssx,ssn)
% bmap = mfb(c,ssx,ssn)
%     c = channel
%     ssx = signal power
%     ssn = noise power
%
%     Calculates the matched filter bound on the achievable
%     bit rate for a given ADSL transmission environment
%
%     Modified from code in the DMT-TEQ toolbox,
%     provided by the University of Texas at Austin, ESPL
%
%     Created by Rick Martin, 06/11/2002
%     Last updated 06/11/2002
%     Cornell University Blind Equalization Research Group

margin = 6;         % dB
codingGain = 4.2;   % dB
snrgap = 9.8;       % dB (loss)
N = 512;            % FFT size
minb = 0;           % minimum number of bits per tone
maxb = 100;         % maximum number of bits per tone
qflag = 1;          % 1: bmap is integer valued

c = c - mean(c);  % ?

% frequency response of channel
Fc_full = fft(c,N);
Fc = Fc_full(1:N/2+1).';

% vector of subchannel SNRs (MFB)
SNRi = (ssx/ssn) * abs(Fc).^2;

bmap = ba_cal(SNRi,9.8,margin,codingGain,minb,maxb,qflag);
